#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_SET_RGB  	 2

uint8_t brightness_R,brightness_G,brightness_B;

void set_RGB(uint8_t R,uint8_t G,uint8_t B);
void out_RGB();

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
}

void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

void set_RGB(uint8_t R,uint8_t G,uint8_t B)
{

	brightness_R = R;	brightness_G = G;	brightness_B = B;
}

void out_RGB()
{
	uint8_t ctr, speed;
	//PORTC=0b00111111;
	//PORTB=0b00101010;
	//return;
	for(ctr = 0; ctr < 255; ctr++)
		{
			//for(speed = 0; speed < 45; speed++)
			//{
				if (ctr < brightness_R)
					PORTC |= 0b00010101;
				else
					PORTC &= 0b00000000;									
				
				if (ctr < brightness_G)
					PORTC |= 0b00101010;
				else
					PORTC &= 0b00000000;									
							
				if (ctr < brightness_B)
					PORTB = 0b00101010;
				else
					PORTB = 0b00000000;									
				//usbPoll();
			//}
		}
}
/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
	else if (rq->bRequest == VENDOR_RQ_SET_RGB)
	{
		uint8_t i;
		//DDRC  = 0b00111111;
		//DDRB  = 0b00101010;
	
	//for(i = 0; ; i++)
		//{ 
			
			/* Shading
			if((i>>7) & 1) brightnessR++;
			if((i>>7) & 1) brightnessG++;
			if((i>>7) & 1) brightnessB++;
			*/
			// brightness
			set_RGB(rq->wIndex.bytes[0], rq->wValue.bytes[1], rq->wValue.bytes[0]);
			//out_RGB();
		//}
	}
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2)|(1<<PC3)|(1<<PC4)|(1<<PC5); // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = (1 << PC3);                 // Enable pull-up at PC3
	
	DDRB  = (1<<PB1)|(1<<PB3)|(1<<PB5);

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){ 
		out_RGB();               /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
